@@ -7,6 +7,7 @@ import operator
 import sys
 import csv
 import chardet
+import pandas
 
 # rk.to_txt()
 def to_txt(l, path = 'text_file.txt', overwrite = False, verbose = False):
@@ -238,6 +239,9 @@ class csv_info:
             self.rawdata = f.read(8192)
 
             self.encoding = chardet.detect(self.rawdata)['encoding']
+            if self.encoding == 'ascii':
+                self.encoding = 'utf8'
+                print('Detected ascii encoding, changed to utf8 for safety')
  
         with open(file, mode='r', encoding=self.encoding) as f:
 
@@ -268,4 +272,74 @@ separator  : {this.separator}
 quotechar  : {this.quotechar}
 encoding   : {this.encoding}
 
-{"{:,}".format(this.rows)} rows, {"{:,}".format(len(this.columns))} columns""")
\ No newline at end of file
+{"{:,}".format(this.rows)} rows, {"{:,}".format(len(this.columns))} columns""")
+
+
+# rk.sql_dict()
+def mysql_dict(con, db_filter = '', table_filter = '', col_filter = '', strict=False) :
+
+#Docstring
+    '''
+    Creates a dictionary listing all databases, tables and columns of a MySql server. Results can be filtered using regex.
+    
+    --------------
+    
+    con : SQLAlchemy connectable (engine/connection) or database string URI or DBAPI2 connection (fallback mode)
+    
+    db_filter : str or list of str, default is ''
+        Filters on databases containing the specified regular expression(s).
+    
+    table_filter : str or list of str, default is ''
+        Filters on tables containing the specified regular expression(s).
+  
+    column_filter : str or list of str, default is ''
+        Filters on columns containing the specified regular expression(s).
+        
+    strict : boolean, default is False
+        Returns only strict matches instead of partial matches.
+        
+    --------------
+
+    Examples :
+    
+    >> temp_dict = sql_dict(con = f'mysql://{user}:{pw}@{host}:{port}', col_filter = 'price')
+    >> temp_dict.keys()
+    dict_keys(['products'])
+    >> temp_dict['products'].keys()
+    dict_keys(['phones', 'laptops'])
+    >> temp_dict['products']['phones']
+    ['price_without_tax',
+     'price_with_tax']
+    '''
+    
+    con_dict = {}
+    
+    filter_dict = {'db_filter' : db_filter, 
+                  'table_filter': table_filter,
+                  'col_filter': col_filter}
+
+    for var in filter_dict:
+        if not isinstance(filter_dict[var], list):
+            filter_dict[var] = [filter_dict[var]]
+        filter_dict[var] = '|'.join("^"*strict + term + "$"*strict for term in filter_dict[var] if term != '')
+    
+    con_df = pd.read_sql(
+f'''
+select table_schema, table_name, column_name from information_schema.columns
+where table_schema regexp '{filter_dict["db_filter"]}'
+and table_name regexp '{filter_dict["table_filter"]}'
+and column_name regexp '{filter_dict["col_filter"]}'
+''', con
+)
+    for db in set(con_df['table_schema']) :
+        db_df = con_df[con_df['table_schema'] == db]
+        db_dict = {}
+        for table in set(db_df['table_name']):
+            table_df = db_df[db_df['table_name']==table]
+            column_list = list(table_df['column_name'])
+            if len(column_list) != 0 :
+                db_dict[table] = column_list
+        if len(db_dict) != 0:
+            con_dict[db] = db_dict
+    
+    return con_dict
\ No newline at end of file
